﻿/*
Copyright © 2012 DMD, LLC.  Government Purpose Rights, ALL OTHER RIGHTS RESERVED
This computer software was developed pursuant to Contract Number W9113M-08-D-0003-0007 with the US
Government. Unauthorized Use, Reproduction, Modification, or Disclosure is strictly prohibited.
*/
var spsUtilities = function () {
    var _getFolderIdByName = function (folderName, path, listName, source) {
        var thisId;
        var camlQueryOptions = "<QueryOptions><Folder>" + $().SPServices.SPGetCurrentSite() + path + "</Folder></QueryOptions>";
        var camlQuery = "<Query><Where><And><Eq><FieldRef Name='FSObjType' /><Value Type='Lookup'>1</Value></Eq><Contains><FieldRef Name='FileRef'/><Value Type='Text'>" + folderName + "</Value></Contains></And></Where></Query>";
        try {
            $().SPServices({
                operation: "GetListItems",
                async: false,
                listName: listName,
                CAMLQuery: camlQuery,
                CAMLQueryOptions: camlQueryOptions,
                completefunc: function (xData, Status) {
                    var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                    if (soapResponse.responseData.IsError) {
                        errorLogData.addError("_getFolderIdByName in spsUtilities.js  Failed", source, soapResponse.responseData.Details);
                        thisId = -1;
                    }
                    else {
                        $(xData.responseXML).find("[nodeName='z:row']").each(function () {
                            thisId = $(this).attr("ows_ID");
                        });
                    }
                }
            });
        } catch (e) {
            errorLogData.addError("_getFolderIdByName in spsUtilities.js  Failed", source, "Exception raised. " + e.name + ": " + e.message);
        }
        if (thisId === undefined) {
            thisId = -1;
        }
        return thisId;
    }, 

	/*
        _getListItems = function (path, listName, source) {
            // gets all items located at 'path' but excludes folders
            var xml = "";
            var camlQueryOptions = "<QueryOptions><Folder>" + $().SPServices.SPGetCurrentSite() + "/Lists/" + listName + "/" + path + "</Folder></QueryOptions>";
            camlQuery = "<Query><Where><Eq><FieldRef Name='FSObjType' /><Value Type='Lookup'>0</Value></Eq></Where></Query>";

            try {
                $().SPServices({
                    operation: "GetListItems",
                    async: false,
                    listName: listName,
                    CAMLQuery: camlQuery,
                    CAMLQueryOptions: camlQueryOptions,
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("getListItems in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                        } else {
                            xml = xData.responseXML;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("getListItems in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
            }
			if (xml === undefined || xml === null) {
                xml = "";
            }
            return xml;
        },
        */
        
        _getListItems = function (path, listName, orderByArray, source) {
            // gets all items located at 'path' but excludes folders - if orderByArray is populated, it will return items in order
            // orderByArray is of the form: items.push(["FieldRef Name", Ascending/Descending]) - ordered the same as order in array list
            var xml = "";
            var camlQueryOptions = "<QueryOptions><Folder>" + $().SPServices.SPGetCurrentSite() + "/Lists/" + listName + "/" + path + "</Folder></QueryOptions>";
            var orderBy = "";

            if (orderByArray.length > 0) {
                orderBy = "<OrderBy>";
                for (var i = 0; i < orderByArray.length; i++) {
                    orderBy += "<FieldRef Name='" + orderByArray[i][1, 0] + "'";
                    if (orderByArray[i][1, 1] === "Descending") {
                        orderBy += " Ascending='False' />";
                    } else {
                        orderBy += " />";
                    }
                }
                orderBy += "</OrderBy>";
            }

            var camlQuery = "<Query><Where><Eq><FieldRef Name='FSObjType' /><Value Type='Lookup'>0</Value></Eq></Where>" + orderBy + "</Query>";

            try {
                $().SPServices({
                    operation: "GetListItems",
                    async: false,
                    listName: listName,
                    CAMLQuery: camlQuery,
                    CAMLQueryOptions: camlQueryOptions,
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("getListItems in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                        } else {
                            xml = xData.responseXML;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("getListItems in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
            }
            if (xml === undefined || xml === null) {
                xml = "";
            }
            return xml;
        },
        
        
        _getDocumentItems = function (path, listName, source) {
            // gets all items located at 'path' but excludes folders
            var xml = "";
            var camlQueryOptions = "<QueryOptions><Folder>" + $().SPServices.SPGetCurrentSite() + "/" + listName + "/" + path + "</Folder></QueryOptions>";
            var camlQuery = "<Query><Where><Eq><FieldRef Name='FSObjType' /><Value Type='Lookup'>0</Value></Eq></Where></Query>";

            try {
                $().SPServices({
                    operation: "GetListItems",
                    async: false,
                    listName: listName,
                    CAMLQuery: camlQuery,
                    CAMLQueryOptions: camlQueryOptions,
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("getListItems in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                        } else {
                            xml = xData.responseXML;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("getListItems in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
            }
            if (xml === undefined || xml === null) {
                xml = "";
            }
            return xml;
        };

    return {

        getCurrentSite: function () {
            return $().SPServices.SPGetCurrentSite();
        },

         getListItems: function (listName, source) {
            var orderByArray = [];
            return _getListItems("", listName, orderByArray, source);
        },

        getListItemsOrderBy: function (listName, orderByArray, source) {
            return _getListItems("", listName, orderByArray, source);
        },

        getListItemsByPath: function (path, listName, source) {
            var orderByArray = [];
            return _getListItems(path, listName, orderByArray, source);
        },

        getListItemsByPathOrderBy: function (path, listName, orderByArray, source) {
            return _getListItems(path, listName, orderByArray, source);
        },
                
        getDocumentItemsByPath: function (path, listName, source) {
            return _getDocumentItems(path, listName, source);
        },
        
        addListItem: function (itemsArray, listName, source) {
        	// path is passed in itemsArray as 'BaseName'
            // if itemsArray contains 'BaseName' there appears to be a problem with adding an item whose Title field is the same as an existing
            // one in the same 'BaseName' path - it will error off and not add the item
            var thisId = -1;

            try {
                $().SPServices({
                    operation: "UpdateListItems",
                    async: false,
                    batchCmd: "New",
                    listName: listName,
                    valuepairs: itemsArray,
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("addListItem in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                            thisId = -1;
                        } else {
                            $(xData.responseXML).find("[nodeName='z:row']").each(function () {
                                thisId = $(this).attr("ows_ID");
                            });
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("addListItem in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
                thisId = -1;
            }
            if (thisId === undefined) {
                thisId = -1;
            }
            return thisId;
        },

        addFolder: function (folderPath, listName, source) {
        	// can add folder to list or document library
            var thisId = -1;

            try {
                $().SPServices({
                    operation: "UpdateListItems",
                    async: false,
                    listName: listName,
                    updates: "<Batch OnError='Continue' PreCalc='TRUE'>" +
                             "<Method ID='1' Cmd='New'>" +
                             "<Field Name='FSObjType'>1</Field>" +
                             "<Field Name='BaseName'>" + folderPath + "</Field>" +
                             "</Method>" +
                             "</Batch>",
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("AddFolder in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                            thisId = -1;
                        } else {
                            $(xData.responseXML).find("[nodeName='z:row']").each(function () {
                                thisId = $(this).attr("ows_ID");
                            });
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("AddFolder in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
                var thisId = -1;
            }
            if (thisId === undefined) {
                thisId = -1;
            }
            return thisId;
        },

        saveListItem: function (itemId, itemsArray, listName, source) {
            var success = true;
            try {
                $().SPServices({
                    operation: "UpdateListItems",
                    async: false,
                    ID: itemId,
                    listName: listName,
                    valuepairs: itemsArray,
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("saveListItem in spsUilities.js Failed", source, soapResponse.responseData.Details);
                            success = false;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("saveListItem in spsUilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
                success = false;
            }
            return success;
        },

        deleteListItem: function (itemId, listName, source) {
        	// should work for an item or a folder
            var deleteSuccess = true;

            try {
                $().SPServices({
                    operation: "UpdateListItems",
                    async: false,
                    listName: listName,
                    updates: "<Batch OnError='Continue' PreCalc='TRUE'>" +
                             "<Method ID='1' Cmd='Delete'>" +
                             "<Field Name='ID'>" + itemId + "</Field>" +
                             "</Method>" +
                             "</Batch>",
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("deleteListItem in spsUilities.js Failed", source, soapResponse.responseData.Details);
                            deleteSuccess = false;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("deleteListItem in spsUilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
                deleteSuccess = false;
            }

            return deleteSuccess;
        },

        deleteDocumentItem: function (documentId, fileRef, listName, source) {
            // fileref is required for documents but not lists
            // should work for an item or a folder
            var deleteSuccess = true;

            var fullFileRef = $().SPServices.SPGetCurrentSite() + fileRef;
            try {
                $().SPServices({
                    operation: "UpdateListItems",
                    async: false,
                    listName: listName,
                    updates: "<Batch OnError='Continue' PreCalc='TRUE'>" +
                             "<Method ID='1' Cmd='Delete'>" +
                             "<Field Name='ID'>" + documentId + "</Field>" +
                             "<Field Name='FileRef'>" + fullFileRef + "</Field>" +
                             "</Method>" +
                             "</Batch>",

                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("deleteDocumentItem in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                            deleteSuccess = false;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("deleteDocumentItem in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
                deleteSuccess = false;
            }

            return deleteSuccess;
        },
        
        getFolderIdByName: function (folderName, path, listName, source) {
            return _getFolderIdByName(folderName, path, listName, source)
        },

        getListItemById: function (itemId, listName, source) {
            // gets item with assigned itemId - returns "" if not found
            var xml = "";
            camlQuery = "<Query><Where><Eq><FieldRef Name='ID' /><Value Type='Counter'>" + itemId + "</Value></Eq></Where></Query>";

            try {
                $().SPServices({
                    operation: "GetListItems",
                    async: false,
                    listName: listName,
                    CAMLQuery: camlQuery,
                    CAMLQueryOptions: "<QueryOptions><ViewAttributes Scope='Recursive'/></QueryOptions>",
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("getListItemById in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                            thisId = -1;
                        } else {
                            xml = xData.responseXML;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("getListItemById in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
            }
            if (xml === undefined || xml === null) {
                xml = "";
            }
            return xml;
        },
        
        getSortedItemsByLookupId: function (lookupId, lookupField, orderByArray, listName, source) {
            // gets items for lookupId with lookupField - returns "" if not found
            // untested for document libraries
            var xml = "";
            
            if (orderByArray.length > 0) {
                orderBy = "<OrderBy>";
                for (var i = 0; i < orderByArray.length; i++) {
                    orderBy += "<FieldRef Name='" + orderByArray[i][1, 0] + "'";
                    if (orderByArray[i][1, 1] === "Descending") {
                        orderBy += " Ascending='False' />";
                    } else {
                        orderBy += " />";
                    }
                }
                orderBy += "</OrderBy>";
            }

            camlQuery = "<Query><Where><Eq><FieldRef Name='" + lookupField + "' LookupId='True' /><Value Type='Lookup'>" + lookupId + "</Value></Eq></Where>" + orderBy+ "</Query>";

            try {
                $().SPServices({
                    operation: "GetListItems",
                    async: false,
                    listName: listName,
                    CAMLQuery: camlQuery,
                    CAMLQueryOptions: "<QueryOptions><ViewAttributes Scope='Recursive'/></QueryOptions>",
                    completefunc: function (xData, Status) {

                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("getItemsByLookupId in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                            thisId = -1;
                        } else {
                            xml = xData.responseXML;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("getItemsByLookupId in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
            }
            if (xml === undefined || xml === null) {
                xml = "";
            }
            return xml;
        },

        getItemsByLookupIdAndLookupFilter: function (lookupId, lookupField, lookupFilterId, lookupFilterField, listName, source) {
            // gets items for lookupId with lookupField and filters by lookupFilterId with lookupFilterField - returns "" if not found
            // untested for document libraries
            var xml = "";
            camlQuery = "<Query><Where><And><Eq><FieldRef Name='" + lookupField + "' LookupId='True' /><Value Type='Lookup'>" + lookupId + "</Value></Eq><Eq><FieldRef Name='" + lookupFilterField + "' LookupId='True' /><Value Type='Lookup'>" + lookupFilterId + "</Value></Eq></And></Where></Query>";

            try {
                $().SPServices({
                    operation: "GetListItems",
                    async: false,
                    listName: listName,
                    CAMLQuery: camlQuery,
                    CAMLQueryOptions: "<QueryOptions><ViewAttributes Scope='Recursive'/></QueryOptions>",
                    completefunc: function (xData, Status) {

                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("getItemsByLookupIdAndLookupFilter in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                            thisId = -1;
                        } else {
                            xml = xData.responseXML;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("getItemsByLookupId in spsUtilities.js Failed", source, "Exception raised. " + e.name + ": " + e.message);
            }
            if (xml === undefined || xml === null) {
                xml = "";
            }
            return xml;
        },

        addMultipleListItems: function (folderPath, listName, itemsArray) {
            // try-catch validation is handled at the SPAddMultipleListItems level
            rootFolderPath = $().SPServices.SPGetCurrentSite() + "/Lists/" + listName + "/" + folderPath;
            // the following code is located in spservicesext.js
            var addSuccess = $().SPServices.SPAddMultipleListItems({
                operation: "UpdateListItems",
                async: false,
                batchCmd: "New",
                rootFolder: rootFolderPath,
                listName: listName,
                items: itemsArray
            });
            return addSuccess;
        },

        copyDocumentsToLocation: function (sourceUrl, destUrl, documentLibrary, source) {
            var success = true,
                thisSite = $().SPServices.SPGetCurrentSite(),
                fullSourceUrl = thisSite + "/" + documentLibrary + "/" + sourceUrl,
                fullDestUrl = thisSite + "/" + documentLibrary + "/" + destUrl;

            try {
                $().SPServices({
                    operation: "CopyIntoItemsLocal",
                    async: false,
                    SourceUrl: fullSourceUrl,
                    DestinationUrls: [fullDestUrl],
                    completefunc: function (xData, Status) {
                        var soapResponse = soapResponseData.getErrorResponse(xData, Status);
                        if (soapResponse.responseData.IsError) {
                            errorLogData.addError("copyDocumentsToLocation in spsUtilities.js Failed", source, soapResponse.responseData.Details);
                            success = false;
                        }
                    }
                });
            } catch (e) {
                errorLogData.addError("copyDocumentToLocation in Contract Action Documents Failed", "copyDocumentToLocation in contractActionDocuments.js", "Exception raised. " + e.name + ": " + e.message);
                success = false;
            }
            return success;
        }
        
    };
} ();
